unit
  TimeStampCounter;
//------------------------------------------------------------------------------

// This unit is perfect for precision time measuring with a resolution of nanoseconds!
// The windows time-functions GetSystemTime, GetLocalTime are to coarse for
// precise time measuring.
//
// GetTSC reads the TSC (Time-Stamp-Counter) of the Pentium processor.
// This 64-bit register counts every processor clock since reset.
// Even with a slow 66 Mhz processor you have a resolution of 15 nanoseconds!
// Who needs less?

// August 2000
// Udo Juerss
// 57078 Siegen
// West Germany

// e-mail: UJHS@aol.com
//------------------------------------------------------------------------------

interface

const
  CalibrateDelayTime = 500; // Time for calibation in ms
  TSCus = 10; // Factor for simplified value input in microseconds
  TSCms = TSCus * 1000; // Factor for simplified value input in milliseconds
//------------------------------------------------------------------------------

var
  CPUSpeed: Double; // contains the CPUSpeed in MHz
                    // You can set this var to your computers speed or call
                    // CalibrateTSC for automatic speed measure
//------------------------------------------------------------------------------

function GetTSC: Int64; assembler; stdcall;
function GetTSCLow: Cardinal;
function GetTSCHigh: Cardinal;
function GetCPUSpeed: Double;
procedure CalibrateTSC;
function GetTSCTicks(Time100nsUnits: Cardinal): Cardinal;
//------------------------------------------------------------------------------

implementation

uses
  Windows;
//------------------------------------------------------------------------------

// Reads the TimeStampCounter as a 64-bit value

function GetTSC: Int64; assembler; stdcall;      // because the inline assembler
asm                  // has no mnemonic for "rdtsc" we have to take the op-codes
                 cli
                 db    0Fh                      // 1. op-code byte for "rdtsc"
                 db    31h                      // 2. op-code byte for "rdtsc"
                 sti
end;
//------------------------------------------------------------------------------

// Reads the lower 32-bit value of the TimeStampCounter

function GetTSCLow: Cardinal;
begin
  Result:=Cardinal(GetTSC);
end;
//------------------------------------------------------------------------------

// Reads the upper 32-bit value of the TimeStampCounter

function GetTSCHigh: Cardinal;
begin
  Result:=GetTSC shr 32;
end;
//------------------------------------------------------------------------------

// Get the amount of TimeStampCounter-ticks for a given time in Time100nsUnits
// For easier use of Time100nsUnits use the constants TSCus and TSCms
// To get the ticks of 750 microseconds: GetTSCTicks(750 * TSCus)
// To get the ticks of 22 milliseconds: GetTSCTicks(22 * TSCms)

function GetTSCTicks(Time100nsUnits: Cardinal): Cardinal;
begin
  Result:=Round(Time100nsUnits * (CPUSpeed / TSCus));
end;
//------------------------------------------------------------------------------

// Calibration routine to compute the processor speed
// If you dont need automatic speed finding, then you can set CPUSpeed to
// your computers clock in MHz.

procedure CalibrateTSC;
var
  StartTime,EndTime: Int64;
  PriorityClass,Priority: Integer;
begin
  PriorityClass:=GetPriorityClass(GetCurrentProcess);
  Priority:=GetThreadPriority(GetCurrentThread);
  SetPriorityClass(GetCurrentProcess,REALTIME_PRIORITY_CLASS);
  SetThreadPriority(GetCurrentThread,{THREAD_PRIORITY_TIME_CRITICAL}31);
  Sleep(TSCus);
  StartTime:=GetTSC;
  Sleep(CalibrateDelayTime);
  EndTime:=GetTSC;
  SetThreadPriority(GetCurrentThread,Priority);
  SetPriorityClass(GetCurrentProcess,PriorityClass);
  CPUSpeed:=(EndTime - StartTime) / (CalibrateDelayTime * 1000.0);
end;
//------------------------------------------------------------------------------

initialization
  CPUSpeed:=500; // Predefined value - use CalribateTSC for automatic speed finding
                 // or set to computer clock in MHz.
end.


